@@ -12,8 +12,6 @@ from typing import Tuple
 from typing import TypeVar
 from typing import Union
 
-import pytz
-
 from certbot import configuration
 from certbot import crypto_util
 from certbot import errors
@@ -117,7 +115,7 @@ def lineage_for_certname(cli_config: configuration.NamespaceConfig,
         return None
     try:
         return storage.RenewableCert(renewal_file, cli_config)
-    except (errors.CertStorageError, IOError):
+    except (OSError, errors.CertStorageError):
         logger.debug("Renewal conf file %s is broken.", renewal_file)
         logger.debug("Traceback was:\n%s", traceback.format_exc())
         return None
@@ -285,7 +283,7 @@ def human_readable_cert_info(config: configuration.NamespaceConfig, cert: storag
         return None
     if config.domains and not set(config.domains).issubset(cert.names()):
         return None
-    now = datetime.datetime.now(pytz.UTC)
+    now = datetime.datetime.now(datetime.timezone.utc)
 
     reasons = []
     if cert.is_test_cert:
@@ -419,7 +417,7 @@ def _search_lineages(cli_config: configuration.NamespaceConfig, func: Callable[.
     for renewal_file in storage.renewal_conf_files(cli_config):
         try:
             candidate_lineage = storage.RenewableCert(renewal_file, cli_config)
-        except (errors.CertStorageError, IOError):
+        except (OSError, errors.CertStorageError):
             logger.debug("Renewal conf file %s is broken. Skipping.", renewal_file)
             logger.debug("Traceback was:\n%s", traceback.format_exc())
             continue
